function codeMirrorDefineX12(){
CodeMirror.defineMode("x12", function(config, parserConfig) {
	var indentUnit = config.indentUnit;
	var jsonMode = parserConfig.json;

	// Tokenizer

	var keywords = function(){
		function kw(type) {return {type: type, style: "keyword"};}
		var operator = kw("operator");
		return {
			"*": operator, "~": operator, "!": operator
		};
	}();

	var isOperatorChar = /[*~!]/;

	function chain(stream, state, f) {
		state.tokenize = f;
		return f(stream, state);
	}

	// Used as scratch variables to communicate multiple values without
	// consing up tons of objects.
	var type, content;
	function ret(tp, style, cont) {
		type = tp; content = cont;
		return style;
	}

	function x12TokenBase(stream, state) {
		var ch = stream.next();
		if (/[\*~]/.test(ch)) {
			return ret(ch, "operator", ch);
		} else if (isOperatorChar.test(ch)) {
			stream.eatWhile(isOperatorChar);
			return ret("operator", "operator", stream.current());
		} else {
			stream.eatWhile(/[\w\$_]/);
			var word = stream.current(), known = keywords.propertyIsEnumerable(word.toLowerCase()) && keywords[word.toLowerCase()];
			return known ? ret(known.type, known.style, word) : ret("variable", "variable", word);
		}
	}

	// Parser

	var atomicTypes = {"atom": true, "variable": true, "string": true, "regexp": true};

	function X12Lexical(indented, column, type, align, prev, info) {
		this.indented = indented;
		this.column = column;
		this.type = type;
		this.prev = prev;
		this.info = info;
		if (align != null) this.align = align;
	}

	function inScope(state, varname) {
		for (var v = state.localVars; v; v = v.next)
			if (v.name == varname) return true;
	}

	function parseX12(state, style, type, content, stream) {
		var cc = state.cc;
		// Communicate our context to the combinators.
		// (Less wasteful than consing up a hundred closures on every call.)
		cx.state = state; cx.stream = stream; cx.marked = null, cx.cc = cc;

		if (!state.lexical.hasOwnProperty("align"))
			state.lexical.align = true;

		while(true) {
			var combinator = cc.length ? cc.pop() : jsonMode ? expression : statement;
			if (combinator(type, content)) {
				while(cc.length && cc[cc.length - 1].lex)
					cc.pop()();
				if (cx.marked) return cx.marked;
				return style;
			}
		}
	}

	// Combinator utils

	var cx = {state: null, column: null, marked: null, cc: null};
	function pass() {
		for (var i = arguments.length - 1; i >= 0; i--) cx.cc.push(arguments[i]);
	}
	function cont() {
		pass.apply(null, arguments);
		return true;
	}
	function register(varname) {
		var state = cx.state;
		if (state.context) {
			cx.marked = "def";
			for (var v = state.localVars; v; v = v.next)
				if (v.name == varname) return;
			state.localVars = {name: varname, next: state.localVars};
		}
	}

	// Combinators

	var defaultVars = {name: "this", next: {name: "arguments"}};
	function pushcontext() {
		if (!cx.state.context) cx.state.localVars = defaultVars;
		cx.state.context = {prev: cx.state.context, vars: cx.state.localVars};
	}
	function popcontext() {
		cx.state.localVars = cx.state.context.vars;
		cx.state.context = cx.state.context.prev;
	}
	function pushlex(type, info) {
		var result = function() {
			var state = cx.state;
			state.lexical = new X12Lexical(state.indented, cx.stream.column(), type, null, state.lexical, info)
		};
		result.lex = true;
		return result;
	}
	function poplex() {
		var state = cx.state;
		if (state.lexical.prev) {
			if (state.lexical.type == ")") {
				state.indented = state.lexical.indented;
			}
			state.lexical = state.lexical.prev;
		}
	}
	poplex.lex = true;

	function expect(wanted) {
		return function expecting(type) {
			if (type == wanted) {
				return cont();
			} else if (wanted == ";") {
				return pass();
			} else {
				return cont(arguments.callee);
			}
		};
	}

	function statement(type) {
		if (type == "var") return cont(pushlex("vardef"), vardef1, expect(";"), poplex);
		if (type == "!") return cont();
		return pass(pushlex("stat"), expression, expect(";"), poplex);
	}
	function expression(type) {
		if (atomicTypes.hasOwnProperty(type)) return cont(maybeoperator);
	    if (type == "function") return cont(functiondef);
	    if (type == "keyword c") return cont(expression);
	    if (type == "operator") return cont(expression);
	    return cont();
	}
	function maybeoperator(type, value) {
		if (type == "operator" && /\+\+|--/.test(value)) return cont(maybeoperator);
		if (type == "operator") return cont(expression);
		if (type == "!") return;
	}
	function maybelabel(type) {
		if (type == ":") return cont(poplex, statement);
		return pass(maybeoperator, expect(";"), poplex);
	}
	function property(type) {
		if (type == "variable") {cx.marked = "property"; return cont();}
	}
	function objprop(type) {
		if (type == "variable") cx.marked = "property";
		if (atomicTypes.hasOwnProperty(type)) return cont(expect(":"), expression);
	}
	function commasep(what, end) {
		function proceed(type) {
			if (type == ",") return cont(what, proceed);
			if (type == end) return cont();
			return cont(expect(end));
		}
		return function commaSeparated(type) {
			if (type == end) return cont();
			else return pass(what, proceed);
		};
	}
	function block(type) {
		if (type == "!") return cont();
		return pass(statement, block);
	}
	function vardef1(type, value) {
		if (type == "variable"){register(value); return cont(vardef2);}
		return cont();
	}
	function vardef2(type, value) {
		if ( (value == "*") || (value == "~") || (value == "!") ) return cont(expression, vardef2);
	}
	function forspec1(type) {
		if ( (type == "!") || (type == "~") ) return cont(vardef1, forspec2);
		if (type == "variable") return cont(formaybein);
		return pass(forspec2);
	}
	function formaybein(type, value) {
		if ((value == "~") || (value == "!")) return cont(expression);
		return cont(maybeoperator, forspec2);
	}
	function forspec2(type, value) {
		if (type == "~") return cont(forspec3);
		return cont(expression, expect("~"), forspec3);
	}
	function forspec3(type) {
		if (type != "~") cont(expression);
	}
	function functiondef(type, value) {
		if (type == "variable") {register(value); return cont(functiondef);}
	}
	function funarg(type, value) {
		if (type == "variable") {register(value); return cont();}
	}

	// Interface

	return {
		startState: function(basecolumn) {
			return {
				tokenize: x12TokenBase,
				reAllowed: true,
				cc: [],
				lexical: new X12Lexical((basecolumn || 0) - indentUnit, 0, "block", false),
				localVars: null,
				context: null,
				indented: 0
			};
		},

		token: function(stream, state) {
			if (stream.sol()) {
				if (!state.lexical.hasOwnProperty("align"))
					state.lexical.align = false;
				state.indented = stream.indentation();
			}
			if (stream.eatSpace()) return null;
			var style = state.tokenize(stream, state);
			if (type == "comment") return style;
			if (style == "variable") {
				var pos = stream.start;
				if (stream.start == 0) {
					style = 'keyword';
					return style;
				}
			}
			state.reAllowed = type == "operator" || type == "keyword c" || type.match(/^[\[{}\(,;:]$/);
			return parseX12(state, style, type, content, stream);
		},

		indent: function(state, textAfter) {
			// No se indenta nada en EdiwinScript. Dejo el cdigo pendiente de revisar por si se quiere implementar la indentacin ms adelante.
			return 0;
		},

		electricChars: ":{}"
	};
});

CodeMirror.defineMIME("text/x-x12", "x12");

}